@@ -3,7 +3,7 @@ package Alien::Base::ModuleBuild;
 use strict;
 use warnings;
 
-our $VERSION = '0.001_003';
+our $VERSION = '0.004_01';
 $VERSION = eval $VERSION;
 
 use parent 'Module::Build';
@@ -15,8 +15,9 @@ use File::Basename qw/fileparse/;
 use Carp;
 use Archive::Extract;
 use Sort::Versions;
-use List::MoreUtils qw/uniq/;
+use List::MoreUtils qw/uniq first_index/;
 use ExtUtils::Installed;
+use File::Copy qw/move/;
 
 use Alien::Base::PkgConfig;
 use Alien::Base::ModuleBuild::Cabinet;
@@ -65,7 +66,7 @@ __PACKAGE__->add_property( alien_selection_method => 'newest' );
 # alien_build_commands: arrayref of commands for building
 __PACKAGE__->add_property( 
   alien_build_commands => 
-  default => [ '%pconfigure --prefix=%s', 'make' ],
+  default => [ '%c --prefix=%s', 'make' ],
 );
 
 # alien_test_commands: arrayref of commands for testing the library
@@ -101,6 +102,9 @@ __PACKAGE__->add_property( 'alien_repository'         => {} );
 __PACKAGE__->add_property( 'alien_repository_default' => {} );
 __PACKAGE__->add_property( 'alien_repository_class'   => {} );
 
+# alien_isolate_dynamic
+__PACKAGE__->add_property( 'alien_isolate_dynamic' => 0 );
+__PACKAGE__->add_property( 'alien_autoconf_with_pic' => 1 );
 
 ################
 #  ConfigData  #
@@ -132,6 +136,17 @@ sub new {
   # setup additional temporary directories, and yes we have to add File::ShareDir manually
   $self->_add_prereq( 'requires', 'File::ShareDir', '1.00' );
 
+  if (grep /(?<!\%)\%c/, @{ $self->alien_build_commands }) {
+    $self->config_data( 'autoconf' => 1 );
+  }
+
+  if ($^O eq 'MSWin32' && $self->config_data( 'autoconf')) {
+    $self->_add_prereq( 'build_requires', 'Alien::MSYS', '0' );
+    $self->config_data( 'msys' => 1 );
+  } else {
+    $self->config_data( 'msys' => 0 );
+  }
+
   # force newest for all automated testing 
   #TODO (this probably should be checked for "input needed" rather than blindly assigned)
   if ($ENV{AUTOMATED_TESTING}) {
@@ -182,12 +197,27 @@ END
 
 sub ACTION_code {
   my $self = shift;
-  $self->depends_on('alien') unless $self->notes('ACTION_alien_completed');
-  $self->SUPER::ACTION_code;
+  $self->notes( 'alien_blib_scheme' => $self->alien_detect_blib_scheme );
+
+  # PLEASE NOTE, BOTH BRANCHES CALL SUPER::ACTION_code !!!!!!!
+  if ( $self->notes('ACTION_alien_completed') ) {
+
+    $self->SUPER::ACTION_code;
+
+  } else {
+
+    $self->depends_on('alien_code');
+    $self->SUPER::ACTION_code;
+
+    # copy the compiled files into blib if running under blib scheme
+    $self->depends_on('alien_install') if $self->notes('alien_blib_scheme');
+  }
 }
 
-sub ACTION_alien {
+sub ACTION_alien_code {
   my $self = shift;
+  local $| = 1; # don't buffer stdout
+
   $self->alien_init_temp_dir;
 
   $self->config_data( name => $self->alien_name );
@@ -217,6 +247,7 @@ sub ACTION_alien {
 
     my $file = $cabinet->files->[0];
     $version = $file->version;
+    $self->config_data( alien_version => $version ); # Temporary setting, may be overridden later
 
     print "Downloading File: " . $file->filename . " ... ";
     my $filename = $file->get;
@@ -231,11 +262,14 @@ sub ACTION_alien {
     $self->config_data( working_directory => $extract_path );
     $CWD = $extract_path;
 
-    print "Building library ... ";
-    unless ($self->alien_do_commands('build')) {
-      print "Failed\n";
-      croak "Build not completed";
+    if ( $file->platform eq 'src' ) {
+      print "Building library ... ";
+      unless ($self->alien_do_commands('build')) {
+        print "Failed\n";
+        croak "Build not completed";
+      }
     }
+
     print "Done\n";
 
   }
@@ -261,6 +295,7 @@ sub ACTION_alien {
 
   # prevent building multiple times (on M::B::dispatch)
   $self->notes( 'ACTION_alien_completed' => 1 );
+
   return;
 }
 
@@ -277,6 +312,15 @@ sub ACTION_test {
 sub ACTION_install {
   my $self = shift;
   $self->SUPER::ACTION_install;
+  $self->depends_on('alien_install');
+}
+
+sub ACTION_alien_install {
+  my $self = shift;
+
+  local $| = 1; # don't buffer stdout
+
+  return if $self->config_data( 'install_type' ) eq 'system';
 
   {
     my $target = $self->alien_library_destination;
@@ -292,10 +336,29 @@ sub ACTION_install {
 
   {
     local $CWD = $self->config_data( 'working_directory' );
-    print "Installing library ... ";
+    print "Installing library to $CWD ... ";
     $self->alien_do_commands('install') or die "Failed\n";
     print "Done\n";
   }
+  
+  if ( $self->alien_isolate_dynamic ) {
+    local $CWD = $self->alien_library_destination;
+    print "Isolating dynamic libraries ... ";
+    mkdir 'dynamic' unless -d 'dynamic';
+    foreach my $dir (qw( bin lib )) {
+      next unless -d $dir;
+      opendir(my $dh, $dir);
+      my @dlls = grep { /\.so/ || /\.(dylib|la|dll|dll\.a)$/ } grep !/^\./, readdir $dh;
+      closedir $dh;
+      foreach my $dll (@dlls) {
+        my $from = File::Spec->catfile($dir, $dll);
+        my $to   = File::Spec->catfile('dynamic', $dll);
+        unlink $to if -e $to;
+        move($from, $to);
+      }
+    }
+    print "Done\n";
+  }
 
   # refresh metadata after library installation
   $self->alien_refresh_manual_pkgconfig( $self->alien_library_destination );
@@ -303,10 +366,18 @@ sub ACTION_install {
 
   # to refresh config_data
   $self->SUPER::ACTION_config_data;
-  $self->SUPER::ACTION_install;
 
-  # refresh the packlist
-  $self->alien_refresh_packlist( $self->alien_library_destination );
+  if ( $self->notes( 'alien_blib_scheme') ) {
+    # reinstall config_data to blib
+    $self->process_files_by_extension('pm');
+
+  } else {
+    # reinstall config_data
+    $self->SUPER::ACTION_install;
+
+    # refresh the packlist
+    $self->alien_refresh_packlist( $self->alien_library_destination );
+  }
 }
 
 #######################
@@ -381,16 +452,59 @@ sub alien_validate_repo {
 
 sub alien_library_destination {
   my $self = shift;
-  my $lib_dir = $self->install_destination('lib');
+
+  # send the library into the blib if running under the blib scheme
+  my $lib_dir = 
+    $self->notes('alien_blib_scheme')
+    ? File::Spec->catdir( $self->base_dir, $self->blib, 'lib' )
+    : $self->install_destination('lib');
+
   my $dist_name = $self->dist_name;
   my $dest = File::Spec->catdir( $lib_dir, qw/auto share dist/, $dist_name );
   return $dest;
 }
 
+# CPAN testers often run tests without installing modules, but rather add
+# blib dirs to @INC, this is a problem, so here we try to deal with it
+sub alien_detect_blib_scheme {
+  my $self = shift;
+
+  return $ENV{ALIEN_BLIB} if defined $ENV{ALIEN_BLIB};
+
+  # check to see if Alien::Base::ModuleBuild is running from blib.
+  # if so it is likely that this is the blib scheme
+
+  (undef, my $dir, undef) = File::Spec->splitpath( __FILE__ );
+  my @dirs = File::Spec->splitdir($dir);
+
+  my $index = first_index { $_ eq 'blib' } @dirs;
+  return 0 if $index == -1;
+
+  if ( $dirs[$index+1] eq 'lib' ) {
+    print qq{'blib' scheme is detected. Setting ALIEN_BLIB=1. If this has been done in error, please set ALIEN_BLIB and restart build process to disambiguate.\n};
+    return 1;
+  }
+
+  carp q{'blib' scheme is suspected, but uncertain. Please set ALIEN_BLIB and restart build process to disambiguate. Setting ALIEN_BLIB=1 for now.};
+  return 1;
+}
+
 ###################
 #  Build Methods  #
 ###################
 
+sub _msys_do_system {
+  my $self = shift;
+  my $command = shift;
+  
+  if ($self->config_data( 'msys' )) {
+    require Alien::MSYS;
+    return Alien::MSYS::msys(sub { $self->do_system( $command ) });
+  }
+  
+  $self->do_system( $command );
+}
+
 sub alien_do_commands {
   my $self = shift;
   my $phase = shift;
@@ -400,7 +514,7 @@ sub alien_do_commands {
 
   foreach my $command (@$commands) {
 
-    my %result = $self->do_system( $command );
+    my %result = $self->_msys_do_system( $command );
     unless ($result{success}) {
       carp "External command ($result{command}) failed! Error: $?\n";
       return 0;
@@ -448,20 +562,34 @@ sub alien_interpolate {
   my ($string) = @_;
 
   my $prefix = $self->alien_exec_prefix;
+  my $configure = $self->alien_configure;
   my $share  = $self->alien_library_destination;
-  my $name   = $self->alien_name;
+  my $name   = $self->alien_name || '';
 
   # substitute:
   #   install location share_dir (placeholder: %s)
   $string =~ s/(?<!\%)\%s/$share/g;
   #   local exec prefix (ph: %p)
   $string =~ s/(?<!\%)\%p/$prefix/g;
+  #   correct incantation for configure on platform
+  $string =~ s/(?<!\%)\%c/$configure/g;
   #   library name (ph: %n)
   $string =~ s/(?<!\%)\%n/$name/g;
   #   current interpreter ($^X) (ph: %x)
   my $perl = $self->perl;
   $string =~ s/(?<!\%)\%x/$perl/g;
 
+  # Version, but only if needed.  Complain if needed and not yet
+  # stored.
+  if ($string =~ /(?<!\%)\%v/) {
+    my $version = $self->config_data( 'alien_version' );
+    if ( ! defined( $version ) ) {
+      carp "Version substution requested but unable to identify";
+    } else {
+      $string =~ s/(?<!\%)\%v/$version/g;
+    }
+  }
+
   #remove escapes (%%)
   $string =~ s/\%(?=\%)//g;
 
@@ -477,6 +605,20 @@ sub alien_exec_prefix {
   }
 }
 
+sub alien_configure {
+  my $self = shift;
+  my $configure;
+  if ($self->config_data( 'msys' )) {
+    $configure = 'sh configure';
+  } else {
+    $configure = './configure';
+  }
+  if ($self->alien_autoconf_with_pic) {
+    $configure .= ' --with-pic';
+  }
+  $configure;
+}
+
 ########################
 #  Post-Build Methods  #
 ########################
@@ -713,7 +855,7 @@ Joel Berger, E<lt>joel.a.berger@gmail.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2012 by Joel Berger
+Copyright (C) 2012-2014 by Joel Berger
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
