@@ -3,7 +3,7 @@ package Alien::Base::ModuleBuild::Repository;
 use strict;
 use warnings;
 
-our $VERSION = '0.001_003';
+our $VERSION = '0.004_01';
 $VERSION = eval $VERSION;
 
 use Carp;
@@ -39,21 +39,31 @@ sub probe {
 
   my $pattern = $self->{pattern};
 
-  my @files = $self->list_files();
+  my @files;
 
-  if ($pattern) {
-    @files = grep { $_ =~ $pattern } @files;
+  if ($self->{exact_filename}) {
+    # if filename provided, use that specific file
+    @files = ($self->{exact_filename});    
+  } else {
+    @files = $self->list_files();
+
+    if ($pattern) {
+	@files = grep { $_ =~ $pattern } @files;
+    }
+
+    carp "Could not find any matching files" unless @files;
   }
 
-  carp "Could not find any matching files" unless @files;
-
   @files = map { +{ 
     repository => $self,
     platform   => $self->{platform},
     filename   => $_,
   } } @files;
 
-  if ($pattern and pattern_has_capture_groups($pattern)) {
+  if ($self->{exact_filename} and $self->{exact_version}) {
+    # if filename and version provided, use a specific version
+    $files[0]->{version} = $self->{exact_version};
+  } elsif ($pattern and pattern_has_capture_groups($pattern)) {
     foreach my $file (@files) {
       $file->{version} = $1 
         if $file->{filename} =~ $pattern;
@@ -70,6 +80,7 @@ sub probe {
 # subclasses are expected to provide 
 sub connection { croak "$_[0] doesn't provide 'connection' method" }
 sub list_files { croak "$_[0] doesn't provide 'list_files' method" }
+# get_file must return filename actually used
 sub get_file  { croak "$_[0] doesn't provide 'get_files' method"  }
 
 1;
